/** @file main.cpp
 *
 * UnXWE Utility - Restores IWAD tag to IWAD files.
 * @author James Haley
 */

/*
 * Copyright(C) 2011 James Haley
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#define _CRT_SECURE_NO_WARNINGS

#include <windows.h>
#include <string>
#include <cstdio>
#include <cstdlib>

using namespace std;

/**
 * Prints a message and then quits the program.
 * @param text Message to place at the console.
 */
static void FatalError(const char *text)
{
   puts(text);
   exit(-1);
}

/**
 * Copies the input file to a backup file before modifying it.
 * @param [in] inname Name of file to copy
 * @return True if successful, false if error.
 */
bool BackupInputFile(const string &inname)
{
   string outname;

   outname = inname + ".unxwe.bak";

   return (CopyFile((LPCTSTR)inname.c_str(), (LPCTSTR)outname.c_str(), FALSE) != 0);
}

/**
 * Fix an IWAD file by changing the first four bytes back to IWAD.
 * @param [in] inname Input file name
 */
void FixIWADFile(const string &inname)
{
   FILE *f;
   char headerbuf[4];

   if(!(f = fopen(inname.c_str(), "r+b")))
      FatalError("Cannot open input file for modification.");

   // check the first 4 bytes; if they are not PWAD, we will not proceed.
   if(fread(headerbuf, 1, 4, f) != 4)
      FatalError("Could not read WAD file header.");
   if(memcmp(headerbuf, "PWAD", 4))
      FatalError("File is not a PWAD, process aborted.");

   // re-position at start
   if(fseek(f, 0, SEEK_SET))
      FatalError("Cannot seek to start of file, process aborted.");

   if(fwrite("IWAD", 1, 4, f) != 4)
      FatalError("Failed to write IWAD header, process aborted.");

   // Done.
   fclose(f);  
}

/**
 * Main program.
 * @param argc Number of arguments
 * @param argv Argument values
 * @return 0 to indicate successful execution; -1 if an error occurs.
 */
int main(int argc, const char *argv[])
{
   if(argc < 2)
      FatalError("Restores a WAD file's header after XWE has ruined it.\n"
                 "Usage: unxwe [infile]");

   string infilename = argv[1];

   // Backup the input file
   if(!BackupInputFile(infilename))
      FatalError("Could not back up input file, process aborted.");

   // Fix the input file
   FixIWADFile(infilename);

   puts("File successfully patched.");

   return 0;
}

// EOF

